@@ -130,8 +130,8 @@ function Mailbox._copy_messages(self, dest, messages)
             self._cached_close(self, self._account._imap)
         end
     else
-        local fast = fetchfast(self._account._imap, self._mailbox, messages)
-        local msgs = fetchmessage(self._account._imap, self._mailbox, messages)
+        local fast = self._fetch_fast(self, messages)
+        local msgs = self._fetch_message(self, messages)
 
         if (ifcore.login(dest._account._imap) ~= true) then
             return
@@ -153,340 +153,7 @@ function Mailbox._copy_messages(self, dest, messages)
 end
 
 
-function Mailbox.check_status(self)
-    if (self._account._login_user(self._account) ~= true) then
-        return
-    end
-
-    local _, exist, recent, unseen = ifcore.status(self._account._imap,
-        self._mailbox)
-
-    if (type(options) == 'table' and options.info == true) then
-        print(string.format("%d messages, %d recent, %d unseen, in %s@%s/%s.",
-                            exist, recent, unseen,
-                            self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return exist, recent, unseen
-end
-
-
-function Mailbox.send_query(self, criteria, charset)
-    _check_optional(criteria, { 'string', 'table' })
-    _check_optional(charset, 'string')
-
-    if (criteria == nil) then
-        criteria = 'ALL'
-    end
-
-    if (charset == nil) then
-        charset = ''
-    end
-
-    if (self._account._login_user(self._account) ~= true) then
-        return
-    end
-
-    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
-        true) then
-        return {}
-    end
-   
-    local query
-    if (type(criteria) == 'string') then
-        query = criteria
-    else 
-        query = _make_query(criteria)
-    end
-
-    local _, results = ifcore.search(self._account._imap, query, charset)
-
-    if (type(options) == 'table' and options.close == true) then
-        self._cached_close(self, self._account._imap)
-    end
-
-    if (results == nil) then
-        return {}
-    end
-
-    local t = {}
-    for n in string.gmatch(results, '%d+') do
-        table.insert(t, tonumber(n))
-    end
-
-    return t
-end
-
-function Mailbox.select_all(self)
-    return self.send_query(self)
-end
-
-
-function Mailbox.add_flags(self, flags, messages)
-    _check_required(flags, 'table')
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', flags, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages flagged in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.remove_flags(self, flags, messages)
-    _check_required(flags, 'table')
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'remove', flags, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages flagged in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.replace_flags(self, flags, messages)
-    _check_required(flags, 'table')
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'replace', flags, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages flagged in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-
-function Mailbox.mark_answered(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', { '\\Answered' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages marked answered in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-
-function Mailbox.mark_deleted(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', { '\\Deleted' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages marked deleted in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.mark_draft(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', { '\\Draft' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages marked draft in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.mark_flagged(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', { '\\Flagged' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages marked flagged in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.mark_seen(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', { '\\Seen' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages marked seen in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.unmark_answered(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'remove', { '\\Answered' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages unmarked answered in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.unmark_deleted(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'remove', { '\\Deleted' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages unmarked deleted in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.unmark_draft(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'remove', { '\\Draft' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages unmarked draft in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.unmark_flagged(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'remove', { '\\Flagged' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages unmarked flagged in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-function Mailbox.unmark_seen(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'remove', { '\\Seen' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages unmarked seen in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-
-function Mailbox.delete_messages(self, messages)
-    _check_required(messages, 'table')
-
-    local r = self._flag_messages(self, 'add', { '\\Deleted' }, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages deleted in %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox))
-    end
-
-    return r
-end
-
-
-function Mailbox.copy_messages(self, dest, messages)
-    _check_required(dest, 'table')
-    _check_required(messages, 'table')
-
-    local r = self._copy_messages(self, dest, messages)
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and r == true) then
-        print(string.format("%d messages copied from %s@%s/%s to %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox,
-                            dest._account._imap.username,
-                            dest._account._imap.server,
-                            dest._mailbox))
-    end
-
-    return r
-end
-
-
-function Mailbox.move_messages(self, dest, messages)
-    _check_required(dest, 'table')
-    _check_required(messages, 'table')
-
-    local rc = self._copy_messages(self, dest, messages)
-    local rf = false
-    if (rc == true) then
-        rf = self._flag_messages(self, 'add', { '\\Deleted' }, messages)
-    end
-
-    if (type(options) == 'table' and options.info == true and
-            messages ~= nil and rc == true and rf == true) then
-        print(string.format("%d messages moved from %s@%s/%s to %s@%s/%s.",
-                            #messages, self._account._imap.username,
-                            self._account._imap.server, self._mailbox,
-                            dest._account._imap.username,
-                            dest._account._imap.server,
-                            dest._mailbox))
-    end
-
-    return rc == true and rf == true
-end
-
-
-function Mailbox.fetch_fast(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_fast(self, messages)
     if (#messages == 0) then
         return
     end
@@ -523,9 +190,7 @@ function Mailbox.fetch_fast(self, messages)
     return results
 end
 
-function Mailbox.fetch_flags(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_flags(self, messages)
     if (#messages == 0) then
         return
     end
@@ -558,9 +223,7 @@ function Mailbox.fetch_flags(self, messages)
     return results
 end
 
-function Mailbox.fetch_date(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_date(self, messages)
     if (#messages == 0) then
         return
     end
@@ -589,9 +252,7 @@ function Mailbox.fetch_date(self, messages)
     return results
 end
 
-function Mailbox.fetch_size(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_size(self, messages)
     if (#messages == 0) then
         return
     end
@@ -620,9 +281,7 @@ function Mailbox.fetch_size(self, messages)
     return results
 end
 
-function Mailbox.fetch_header(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_header(self, messages)
     if (#messages == 0) then
         return
     end
@@ -657,9 +316,7 @@ function Mailbox.fetch_header(self, messages)
     return results
 end
 
-function Mailbox.fetch_body(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_body(self, messages)
     if (#messages == 0) then
         return
     end
@@ -693,15 +350,13 @@ function Mailbox.fetch_body(self, messages)
     return results
 end
 
-function Mailbox.fetch_message(self, messages)
-    _check_required(messages, 'table')
-
+function Mailbox._fetch_message(self, messages)
     if (#messages == 0) then
         return
     end
 
-    local header = self.fetch_header(self, messages)
-    local body = self.fetch_body(self, messages)
+    local header = self._fetch_header(self, messages)
+    local body = self._fetch_body(self, messages)
 
     local results = {}
     for _, m in ipairs(messages) do
@@ -711,10 +366,8 @@ function Mailbox.fetch_message(self, messages)
     return results
 end
 
-function Mailbox.fetch_fields(self, fields, messages)
-    _check_required(fields, 'table')
-    _check_required(messages, 'table')
 
+function Mailbox._fetch_fields(self, fields, messages)
     if (#messages == 0) then
         return
     end
@@ -753,6 +406,419 @@ function Mailbox.fetch_fields(self, fields, messages)
     return results
 end
 
+function Mailbox.check_status(self)
+    if (self._account._login_user(self._account) ~= true) then
+        return
+    end
+
+    local _, exist, recent, unseen = ifcore.status(self._account._imap,
+        self._mailbox)
+
+    if (type(options) == 'table' and options.info == true) then
+        print(string.format("%d messages, %d recent, %d unseen, in %s@%s/%s.",
+                            exist, recent, unseen,
+                            self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return exist, recent, unseen
+end
+
+
+function Mailbox.send_query(self, criteria, charset)
+    _check_optional(criteria, { 'string', 'table' })
+    _check_optional(charset, 'string')
+
+    if (criteria == nil) then
+        criteria = ''
+    end
+
+    if (charset == nil) then
+        charset = ''
+    end
+
+    if (self._account._login_user(self._account) ~= true) then
+        return
+    end
+
+    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
+        true) then
+        return {}
+    end
+   
+    local query
+    if (type(criteria) == 'string') then
+        query = 'ALL ' .. criteria
+    else 
+        query = _make_query(criteria)
+    end
+
+    local _, results = ifcore.search(self._account._imap, query, charset)
+
+    if (type(options) == 'table' and options.close == true) then
+        self._cached_close(self, self._account._imap)
+    end
+
+    if (results == nil) then
+        return {}
+    end
+
+    local t = {}
+    for n in string.gmatch(results, '%d+') do
+        table.insert(t, tonumber(n))
+    end
+
+    return t
+end
+
+function Mailbox.select_all(self)
+    return Set(self.send_query(self))
+end
+
+
+function Mailbox.add_flags(self, flags, messages)
+    _check_required(flags, 'table')
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', flags, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages flagged in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.remove_flags(self, flags, messages)
+    _check_required(flags, 'table')
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'remove', flags, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages flagged in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.replace_flags(self, flags, messages)
+    _check_required(flags, 'table')
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'replace', flags, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages flagged in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+
+function Mailbox.mark_answered(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', { '\\Answered' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages marked answered in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+
+function Mailbox.mark_deleted(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', { '\\Deleted' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages marked deleted in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.mark_draft(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', { '\\Draft' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages marked draft in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.mark_flagged(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', { '\\Flagged' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages marked flagged in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.mark_seen(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', { '\\Seen' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages marked seen in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.unmark_answered(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'remove', { '\\Answered' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages unmarked answered in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.unmark_deleted(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'remove', { '\\Deleted' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages unmarked deleted in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.unmark_draft(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'remove', { '\\Draft' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages unmarked draft in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.unmark_flagged(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'remove', { '\\Flagged' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages unmarked flagged in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+function Mailbox.unmark_seen(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'remove', { '\\Seen' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages unmarked seen in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+
+function Mailbox.delete_messages(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._flag_messages(self, 'add', { '\\Deleted' }, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages deleted in %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox))
+    end
+
+    return r
+end
+
+
+function Mailbox.copy_messages(self, dest, messages)
+    _check_required(dest, 'table')
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local r = self._copy_messages(self, dest, mesgs)
+
+    if (type(options) == 'table' and options.info == true and r == true) then
+        print(string.format("%d messages copied from %s@%s/%s to %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox,
+                            dest._account._imap.username,
+                            dest._account._imap.server,
+                            dest._mailbox))
+    end
+
+    return r
+end
+
+
+function Mailbox.move_messages(self, dest, messages)
+    _check_required(dest, 'table')
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    local rc = self._copy_messages(self, dest, mesgs)
+    local rf = false
+    if (rc == true) then
+        rf = self._flag_messages(self, 'add', { '\\Deleted' }, mesgs)
+    end
+
+    if (type(options) == 'table' and options.info == true and
+        rc == true and rf == true) then
+        print(string.format("%d messages moved from %s@%s/%s to %s@%s/%s.",
+                            #mesgs, self._account._imap.username,
+                            self._account._imap.server, self._mailbox,
+                            dest._account._imap.username,
+                            dest._account._imap.server,
+                            dest._mailbox))
+    end
+
+    return rc == true and rf == true
+end
+
+
+function Mailbox.fetch_fast(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_fast(self, mesgs)
+end
+
+function Mailbox.fetch_flags(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_flags(self, mesgs)
+end
+
+function Mailbox.fetch_date(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_date(self, mesgs)
+end
+
+function Mailbox.fetch_size(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_size(self, mesgs)
+end
+
+function Mailbox.fetch_header(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_header(self, mesgs)
+end
+
+function Mailbox.fetch_body(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_body(self, mesgs)
+end
+
+function Mailbox.fetch_message(self, messages)
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_message(self, mesgs)
+end
+
+function Mailbox.fetch_fields(self, fields, messages)
+    _check_required(fields, 'table')
+    _check_required(messages, 'table')
+
+    local mesgs = _make_normal(messages)
+
+    return self._fetch_fields(self, fields, mesgs)
+end
+
 
 function Mailbox.is_answered(self)
     return Set(self.send_query(self, 'ANSWERED'))
@@ -931,8 +997,13 @@ function Mailbox.match_field(self, field, pattern)
     _check_required(pattern, 'string')
 
     local messages = self.send_query(self)
-    local fields = self.fetch_fields(self, { field }, messages)
+    local mesgs = _make_normal(messages)
+    local fields = self._fetch_fields(self, { field }, mesgs)
 
+    if (#mesgs == 0 or fields == nil) then
+        return
+    end
+ 
     local results = {}
     for m, f in ipairs(fields) do
         if (regex_search(pattern, f)) then
@@ -977,9 +1048,10 @@ function Mailbox.match_header(self, pattern)
     _check_required(pattern, 'string')
 
     local messages = self.send_query(self)
-    local header = self.fetch_header(self, messages)
+    local mesgs = _make_normal(messages)
+    local header = self._fetch_header(self, mesgs)
 
-    if (#messages == 0 or header == nil) then
+    if (#mesgs == 0 or header == nil) then
         return
     end
 
@@ -997,7 +1069,12 @@ function Mailbox.match_body(self, pattern)
     _check_required(pattern, 'string')
 
     local messages = self.send_query(self)
-    local body = self.fetch_body(self, messages)
+    local mesgs = _make_normal(messages)
+    local body = self._fetch_body(self, mesgs)
+
+    if (#mesgs == 0 or body == nil) then
+        return
+    end
 
     local results = {}
     for m, b in ipairs(body) do
@@ -1013,7 +1090,12 @@ function Mailbox.match_message(self, pattern)
     _check_required(pattern, 'string')
 
     local messages = self.send_query(self)
-    local full = self.fetch_message(self, messages)
+    local mesgs = _make_normal(messages)
+    local full = self._fetch_message(self, mesgs)
+
+    if (#mesgs == 0 or full == nil) then
+        return
+    end
 
     local results = {}
     for m, f in ipairs(full) do
